import gtk, gtk.glade
import os.path
import sys

sys.path.append(os.path.join('..', '..'))
import infoviz

class Plugin(gtk.Widget):
    def __init__(self, widget, controller):
        self.controller = controller
        xml = gtk.glade.XML(os.path.join('plugins', 'fsbrowser', 'fsbrowser.glade'), "vbox1")
        self.vbox = xml.get_widget('vbox1')
        widget.pack_start(self.vbox)
        xml.signal_autoconnect({'on_button1_clicked' : self._on_button1_clicked})
        self.root = infoviz.Node()
        self.root.text = ""
        self.root.id = 0
        self.node_table = {}
        self.counter = 0
        self.lastnode = self.root

    def get_widget(self):
        return self.vbox

    def _on_button1_clicked(self, obj):
        def on_ok(button, self, fsel):
            self.recurse(None, fsel.get_filename())
            fsel.destroy()
            
            self.controller.node_table = self.node_table            
            self.controller.refresh_data()
            
            
        def on_cancel(button, fsel):
            fsel.destroy()
        
        fsel = gtk.FileSelection("Select the directory root:")
        fsel.set_modal(gtk.TRUE)
        fsel.ok_button.connect("clicked", on_ok, self, fsel)
        fsel.cancel_button.connect("clicked", on_cancel, fsel)
        fsel.show()

    def recurse(self, lastnode, path):
        n = infoviz.Node()
        n.text = path
        n.id = self.counter
        self.counter += 1
        self.node_table[n] = []
        
        if lastnode:        
            self.node_table[lastnode].append(n)
            self.node_table[n].append(lastnode)
        else:
            self.controller.selected = n
            
        try:
            for dir in [file for file in os.listdir(path) if os.path.isdir(os.path.join(path, file)) and not os.path.islink(os.path.join(path, file))]:
                self.recurse(n, os.path.join(path, dir))
        except OSError:
            pass

def init(widget, controller):
    return Plugin(widget, controller)
