import gtk, gtk.glade
import os.path
import sys

sys.path.append(os.path.join('..', '..'))
import infoviz

class Plugin(gtk.Widget):
    def __init__(self, widget, controller):
        self.controller = controller
        xml = gtk.glade.XML(os.path.join('plugins', 'makefile', 'makefile.glade'), "vbox1")
        self.vbox = xml.get_widget('vbox1')
        widget.pack_start(self.vbox)
        xml.signal_autoconnect({'on_button1_clicked' : self._on_button1_clicked})

    def get_widget(self):
        return self.vbox

    def _on_button1_clicked(self, obj):
        def on_ok(button, self, fsel):
            self.parse(fsel.get_filename())
            fsel.destroy()
            
        def on_cancel(button, fsel):
            fsel.destroy()
        
        fsel = gtk.FileSelection("Open a Makefile:")
        fsel.set_modal(gtk.TRUE)
        fsel.ok_button.connect("clicked", on_ok, self, fsel)
        fsel.cancel_button.connect("clicked", on_cancel, fsel)
        fsel.show()

    def parse(self, filename):
        root = infoviz.Node()
        root.text = ""
        root.id = 0
        node_table = {root : []}

        counter = 1
        fd = open(filename)
        for line in fd.readlines():
            if line[0] != '\t' and line[0] != '#' and line.find('=') == -1 and line.find(':') != -1:
                line = " ".join(line.split('#')[0].split(':')).split()
                for l in line:
                    found = 0
                    for n in node_table.keys():
                        if n.text == l:
                            found = 1
                            break

                    if not found:
                        n = infoviz.Node()
                        n.id = counter
                        counter += 1
                        n.text = l
                        node_table[n] = []

        fd.seek(0)
        # 2nd pass
        for line in fd.readlines():
            if line[0] != '\t' and line[0] != '#' and line.find('=') == -1 and line.find(':') != -1:
                targets = line.split(":")[0].split()
                line = " ".join(line.split('#')[0].split(':')).split()
                target_nodes = [node for node in node_table.keys() if node.text in targets]
                for target_node in target_nodes:
                    if node_table[target_node] == []:
                        node_table[root].append(target_node)
                        node_table[target_node].append(root)
                    
                    for l in line[len(targets):]:
                        l_node = [node for node in node_table.keys() if node.text == l]
                        node_table[l_node[0]].append(target_node)
                        node_table[target_node].append(l_node[0])

        fd.close()
        
        self.controller.selected = root # node_table.keys()[0]
        self.controller.node_table = node_table        
        self.controller.refresh_data()

def init(widget, controller):
    return Plugin(widget, controller)
