import gtk, gtk.glade
import os.path
import sys

from xml.sax import saxlib
from xml.sax import saxexts

sys.path.append(os.path.join('..', '..'))
import infoviz

class Plugin(gtk.Widget, saxlib.HandlerBase):
    def __init__(self, widget, controller):
        self.controller = controller
        xml = gtk.glade.XML(os.path.join('plugins', 'orgchart', 'orgchart.glade'), "vbox1")
        self.vbox = xml.get_widget('vbox1')
        widget.pack_start(self.vbox)
        xml.signal_autoconnect({'on_button1_clicked' : self._on_button1_clicked})

        # create a parser
        self.parser = saxexts.make_parser()

        # tell the parser to use our handler and error handler
        self.parser.setDocumentHandler(self)
        self.parser.setErrorHandler(self)

    def get_widget(self):
        return self.vbox
        
    def __normalize_whitespace(self, text):
        """remove redundant whitespace from a string"""
        return " ".join(text.split())

    def startElement(self, tag, attrs):
        if tag == "node":
            self.node_counter += 1
            self.counter += 1
            
            n = infoviz.Node()
            n.id = self.counter
            n.text = self.__normalize_whitespace(attrs.get("text", "") )

            self.node_table[n] = [self.stack[self.node_counter-1]]
            self.node_table[self.stack[self.node_counter-1]].append(n)

            if len(self.stack) == self.node_counter:
                self.stack.append(n)
            else:
                self.stack[self.node_counter] = n

    def endElement(self, tag):
        if tag == 'node':
            self.node_counter -= 1
            self.stack.pop()

    def error(self, exception):
        raise exception
    
    def fatalError(self, exception):
        raise exception


    def _on_button1_clicked(self, obj):
        def on_ok(button, self, fsel):
            self.root = infoviz.Node()
            self.root.text = ""
            self.root.id = 0
            self.node_table = {self.root : []}
            self.node_counter = 0
            self.counter = 1
            self.stack = [self.root]

            fd = open(fsel.get_filename())
            self.parser.parseFile(fd)
            self.parser.close()
            fd.close()
            fsel.destroy()
            
            self.controller.node_table = self.node_table
            self.controller.selected = self.root
            self.controller.refresh_data()
            
        def on_cancel(button, fsel):
            fsel.destroy()
        
        fsel = gtk.FileSelection("Select the xml file:")
        fsel.set_modal(gtk.TRUE)
        fsel.ok_button.connect("clicked", on_ok, self, fsel)
        fsel.cancel_button.connect("clicked", on_cancel, fsel)
        fsel.show()

def init(widget, controller):
    return Plugin(widget, controller)
